@@ -17,37 +17,24 @@ import hmac
 import logging
 import random
 import re
-import string
-import sys
 import time
-from cStringIO import StringIO
-
-if sys.version_info[:2] >= (2, 5):
-    from hashlib import sha1
-    from email.utils import formatdate
-else:
-    import sha as sha1
-    from email.Utils import formatdate
+from hashlib import sha1
+from email.utils import formatdate
 
 import zope.location
 from persistent import Persistent
 from zope import schema, component
-from zope.interface import implements
+from zope.interface import implementer
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.publisher.interfaces.http import IHTTPApplicationRequest
 from zope.i18nmessageid import ZopeMessageFactory as _
-from zope.session.interfaces import IClientIdManager
 from zope.schema.fieldproperty import FieldProperty
 
-__docformat__ = 'restructuredtext'
+from zope.session.interfaces import IClientIdManager
+from zope.session.session import digestEncode
 
-cookieSafeTrans = string.maketrans("+/", "-.")
 
-logger = logging.getLogger()
-
-def digestEncode(s):
-    """Encode SHA digest for cookie."""
-    return s.encode("base64")[:-2].translate(cookieSafeTrans)
+logger = logging.getLogger(__name__)
 
 class MissingClientIdException(Exception):
     """No ClientId found in Request"""
@@ -130,10 +117,10 @@ class ICookieClientIdManager(IClientIdManager):
         )
 
 
+@implementer(IClientIdManager, ICookieClientIdManager)
 class CookieClientIdManager(zope.location.Location, Persistent):
     """Session utility implemented using cookies."""
 
-    implements(IClientIdManager, ICookieClientIdManager)
 
     thirdparty = FieldProperty(ICookieClientIdManager['thirdparty'])
     cookieLifetime = FieldProperty(ICookieClientIdManager['cookieLifetime'])
@@ -186,9 +173,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         if namespace is None:
             namespace = "zope3_cs_%x" % (int(time.time()) - 1000000000)
         if secret is None:
-            secret = '%.20f' % random.random()
-        else:
-            secret = str(secret)
+            secret = u'%.20f' % random.random()
         self.namespace = namespace
         self.secret = secret
 
@@ -197,8 +182,9 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         This creates one if necessary:
 
+          >>> from io import StringIO
           >>> from zope.publisher.http import HTTPRequest
-          >>> request = HTTPRequest(StringIO(''), {})
+          >>> request = HTTPRequest(StringIO(), {})
           >>> bim = CookieClientIdManager()
           >>> id = bim.getClientId(request)
           >>> id == bim.getClientId(request)
@@ -206,7 +192,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         The id is retained accross requests:
 
-          >>> request2 = HTTPRequest(StringIO(''), {})
+          >>> request2 = HTTPRequest(StringIO(), {})
           >>> request2._cookies = dict(
           ...   [(name, cookie['value'])
           ...    for (name, cookie) in request.response._cookies.items()
@@ -220,7 +206,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         an IClientId. This is because this method is used to implement
         the IClientId Adapter.
 
-          >>> type(id) == type('')
+          >>> type(id) == type(u'')
           True
 
         We don't set the client id unless we need to, so, for example,
@@ -244,16 +230,16 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         will only be set on POST requests.
 
           >>> bim.postOnly = True
-          >>> request = HTTPRequest(StringIO(''), {})
+          >>> request = HTTPRequest(StringIO(), {})
           >>> bim.getClientId(request)
           Traceback (most recent call last):
           ...
           MissingClientIdException
 
-          >>> print request.response.getCookie(bim.namespace)
+          >>> print(request.response.getCookie(bim.namespace))
           None
 
-          >>> request = HTTPRequest(StringIO(''), {'REQUEST_METHOD': 'POST'})
+          >>> request = HTTPRequest(StringIO(), {'REQUEST_METHOD': 'POST'})
           >>> id = bim.getClientId(request)
           >>> id == bim.getClientId(request)
           True
@@ -269,13 +255,13 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         not set a cookie.
 
           >>> bim.thirdparty = True
-          >>> request = HTTPRequest(StringIO(''), {})
+          >>> request = HTTPRequest(StringIO(), {})
           >>> bim.getClientId(request)
           Traceback (most recent call last):
           ...
           MissingClientIdException
 
-          >>> print request.response.getCookie(bim.namespace)
+          >>> print(request.response.getCookie(bim.namespace))
           None
 
         """
@@ -307,16 +293,12 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         """
         data = "%.20f%.20f%.20f" % (random.random(), time.time(), time.clock())
-        # BBB code for Python 2.4, inspired by the fallback in hmac
-        if hasattr(sha1, '__call__'):
-            digest = sha1(data).digest()
-        else:
-            digest = sha1.new(data).digest()
+        digest = sha1(data.encode()).digest()
         s = digestEncode(digest)
         # we store a HMAC of the random value together with it, which makes
         # our session ids unforgeable.
-        mac = hmac.new(self.secret, s, digestmod=sha1).digest()
-        return s + digestEncode(mac)
+        mac = hmac.new(self.secret.encode(), s, digestmod=sha1).digest()
+        return (s + digestEncode(mac)).decode()
 
     def getRequestId(self, request):
         """Return the browser id encoded in request as a string
@@ -325,8 +307,9 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         For example:
 
+          >>> from io import StringIO
           >>> from zope.publisher.http import HTTPRequest
-          >>> request = HTTPRequest(StringIO(''), {}, None)
+          >>> request = HTTPRequest(StringIO(), {}, None)
           >>> bim = CookieClientIdManager()
 
         Because no cookie has been set, we get no id:
@@ -347,7 +330,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         When we set the request id, we also set a response cookie.  We
         can simulate getting this cookie back in a subsequent request:
 
-          >>> request2 = HTTPRequest(StringIO(''), {}, None)
+          >>> request2 = HTTPRequest(StringIO(), {}, None)
           >>> request2._cookies = dict(
           ...   [(name, cookie['value'])
           ...    for (name, cookie) in request.response._cookies.items()
@@ -361,7 +344,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         Test a corner case where Python 2.6 hmac module does not allow
         unicode as input:
 
-          >>> id_uni = unicode(bim.generateUniqueId())
+          >>> id_uni = bim.generateUniqueId()
           >>> bim.setRequestId(request, id_uni)
           >>> bim.getRequestId(request) == id_uni
           True
@@ -371,7 +354,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
           >>> bim.namespace = 'uid'
           >>> bim.thirdparty = True
-          >>> request3 = HTTPRequest(StringIO(''), {}, None)
+          >>> request3 = HTTPRequest(StringIO(), {}, None)
           >>> request3._cookies = {'uid': 'AQAAf0Y4gjgAAAQ3AwMEAg=='}
           >>> bim.getRequestId(request3)
           'AQAAf0Y4gjgAAAQ3AwMEAg=='
@@ -399,8 +382,8 @@ class CookieClientIdManager(zope.location.Location, Persistent):
             # call encode() on value s a workaround a bug where the hmac
             # module only accepts str() types in Python 2.6
             if (digestEncode(hmac.new(
-                    self.secret, s.encode(), digestmod=sha1
-                ).digest()) != mac):
+                    self.secret.encode(), s.encode(), digestmod=sha1
+                ).digest()).decode() != mac):
                 return None
             else:
                 return sid
@@ -415,8 +398,9 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         Note that the id is checked for validity. Setting an
         invalid value is silently ignored:
 
+            >>> from io import StringIO
             >>> from zope.publisher.http import HTTPRequest
-            >>> request = HTTPRequest(StringIO(''), {}, None)
+            >>> request = HTTPRequest(StringIO(), {}, None)
             >>> bim = CookieClientIdManager()
             >>> bim.getRequestId(request)
             >>> bim.setRequestId(request, 'invalid id')
@@ -430,13 +414,13 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         By default, session cookies don't expire:
 
-            >>> cookie.has_key('expires')
+            >>> 'expires' in cookie
             False
 
         Expiry time of 0 means never (well - close enough)
 
             >>> bim.cookieLifetime = 0
-            >>> request = HTTPRequest(StringIO(''), {}, None)
+            >>> request = HTTPRequest(StringIO(), {}, None)
             >>> bid = bim.getClientId(request)
             >>> cookie = request.response.getCookie(bim.namespace)
             >>> cookie['expires']
@@ -445,20 +429,28 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         A non-zero value means to expire after than number of seconds:
 
             >>> bim.cookieLifetime = 3600
-            >>> request = HTTPRequest(StringIO(''), {}, None)
+            >>> request = HTTPRequest(StringIO(), {}, None)
             >>> bid = bim.getClientId(request)
             >>> cookie = request.response.getCookie(bim.namespace)
-            >>> import rfc822
-            >>> expires = time.mktime(rfc822.parsedate(cookie['expires']))
-            >>> expires > time.mktime(time.gmtime()) + 55*60
+            >>> import email.utils
+            >>> c_expires = email.utils.parsedate(cookie['expires'])
+            >>> from datetime import datetime, timedelta
+            >>> expires = datetime(*c_expires[:7])
+            >>> now = datetime.utcnow()
+            >>> expires > now + timedelta(minutes=55)
             True
 
         If another server in front of Zope (Apache, Nginx) is managing the
         cookies we won't set any ClientId cookies:
 
-          >>> request = HTTPRequest(StringIO(''), {}, None)
+          >>> request = HTTPRequest(StringIO(), {}, None)
           >>> bim.thirdparty = True
-          >>> bim.setRequestId(request, '1234')
+          >>> from zope.testing.loggingsupport import InstalledHandler
+          >>> handler = InstalledHandler(__name__)
+          >>> bim.setRequestId(request, '2345')
+          >>> handler.uninstall()
+          >>> len(handler.records)
+          1
           >>> cookie = request.response.getCookie(bim.namespace)
           >>> cookie
 
@@ -467,18 +459,20 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
           >>> bim.thirdparty = False
           >>> bim.cookieLifetime = None
-          >>> request = HTTPRequest(StringIO(''), {}, None)
+          >>> request = HTTPRequest(StringIO(), {}, None)
           >>> bim.secure = True
           >>> bim.setRequestId(request, '1234')
-          >>> print request.response.getCookie(bim.namespace)
+          >>> from pprint import pprint
+          >>> pprint(request.response.getCookie(bim.namespace))
           {'path': '/', 'secure': True, 'value': '1234'}
 
         If the domain is specified, it will be set as a cookie attribute.
 
           >>> bim.domain = u'.example.org'
           >>> bim.setRequestId(request, '1234')
-          >>> print request.response.getCookie(bim.namespace)
-          {'path': '/', 'domain': u'.example.org', 'secure': True, 'value': '1234'}
+          >>> pprint(request.response.getCookie(bim.namespace))
+          {'domain': '.example.org', 'path': '/', 'secure': True,
+           'value': '1234'}
 
         When the cookie is set, cache headers are added to the
         response to try to prevent the cookie header from being cached:
@@ -493,12 +487,13 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         If the httpOnly attribute is set to a true value, then the
         HttpOnly cookie option is included.
 
-          >>> request = HTTPRequest(StringIO(''), {}, None)
+          >>> request = HTTPRequest(StringIO(), {}, None)
           >>> bim.secure = False
           >>> bim.httpOnly = True
           >>> bim.setRequestId(request, '1234')
-          >>> print request.response.getCookie(bim.namespace)
-          {'path': '/', 'domain': u'.example.org', 'value': '1234', 'httponly': True}
+          >>> pprint(request.response.getCookie(bim.namespace))
+          {'domain': '.example.org', 'httponly': True, 'path': '/',
+           'value': '1234'}
 
         """
         # TODO: Currently, the path is the ApplicationURL. This is reasonable,
@@ -551,8 +546,9 @@ def notifyVirtualHostChanged(event):
     setRequestId if a cookie is present in the response for that manager. To
     demonstrate we create a dummy manager object and event:
 
-        >>> class DummyManager(object):
-        ...     implements(ICookieClientIdManager)
+        >>> from io import StringIO
+        >>> @implementer(ICookieClientIdManager)
+        ... class DummyManager(object):
         ...     namespace = 'foo'
         ...     thirdparty = False
         ...     request_id = None
@@ -563,7 +559,7 @@ def notifyVirtualHostChanged(event):
         >>> component.provideUtility(manager, IClientIdManager)
         >>> from zope.publisher.http import HTTPRequest
         >>> class DummyEvent (object):
-        ...     request = HTTPRequest(StringIO(''), {}, None)
+        ...     request = HTTPRequest(StringIO(), {}, None)
         >>> event = DummyEvent()
 
     With no cookies present, the manager should not be called:
