@@ -13,9 +13,8 @@
 ##############################################################################
 """Session implementation
 """
-from cStringIO import StringIO
-import time, string, random, thread
-from UserDict import IterableUserDict
+import random
+import time
 from heapq import heapify, heappop
 
 import ZODB
@@ -34,19 +33,54 @@ from zope.session.interfaces import \
         IClientIdManager, IClientId, ISession, ISessionDataContainer, \
         ISessionPkgData, ISessionData
 
-__docformat__ = 'restructuredtext'
+try:
+    from cStringIO import StringIO
+except ImportError:
+    # Py3: Now StringIO location.
+    from io import StringIO
 
-cookieSafeTrans = string.maketrans("+/", "-.")
+try:
+    from UserDict import IterableUserDict as UserDict
+except ImportError:
+    # Py3: Now StringIO location.
+    from collections import UserDict
+
+try:
+    from base64 import encodebytes
+except ImportError:
+    # Py3: Python 2 only has encodestring
+    from base64 import encodestring as encodebytes
+
+try:
+    from threading import get_ident
+except ImportError:
+    # Py3: Python 2 has a different location.
+    from thread import get_ident
+
+try:
+    unicode
+except NameError:
+    # Py3: Define unicode
+    unicode = str
+
+try:
+    transtable = bytes.maketrans(b'+/', b'-.')
+except AttributeError:
+    # Py3: Python 2 has a differnt location for maketrans.
+    import string
+    transtable = string.maketrans(b'+/', b'-.')
 
 def digestEncode(s):
     """Encode SHA digest for cookie."""
-    return s.encode("base64")[:-2].translate(cookieSafeTrans)
+    return encodebytes(s)[:-2].translate(transtable)
 
 
-class ClientId(str):
+@zope.interface.implementer(IClientId)
+@zope.component.adapter(IRequest)
+class ClientId(unicode):
     """See zope.session.interfaces.IClientId
 
-        >>> import tests
+        >>> from zope.session import tests
         >>> request = tests.setUp()
 
         >>> id1 = ClientId(request)
@@ -57,21 +91,18 @@ class ClientId(str):
         >>> tests.tearDown()
 
     """
-    zope.interface.implements(IClientId)
-    zope.component.adapts(IRequest)
 
     def __new__(cls, request):
-        return str.__new__(cls,
+        return unicode.__new__(cls,
             zope.component.getUtility(IClientIdManager).getClientId(request)
             )
 
 
-class PersistentSessionDataContainer(zope.location.Location,
-    persistent.Persistent,
-    IterableUserDict):
+@zope.interface.implementer(ISessionDataContainer)
+class PersistentSessionDataContainer(
+    zope.location.Location, persistent.Persistent, UserDict):
     """A SessionDataContainer that stores data in the ZODB"""
 
-    zope.interface.implements(ISessionDataContainer)
 
     _v_last_sweep = 0 # Epoch time sweep last run
     disable_implicit_sweeps = False
@@ -116,8 +147,8 @@ class PersistentSessionDataContainer(zope.location.Location,
 
         But the data is not automatically removed.
 
-            >>> sdc['stale'] #doctest: +ELLIPSIS
-            <zope.session.session.SessionData object at ...>
+            >>> sdc['stale'] is stale
+            True
 
         We can manually remove stale data by calling sweep() if stale
         data isn't being automatically removed.
@@ -212,9 +243,14 @@ class PersistentSessionDataContainer(zope.location.Location,
                 [...]
             KeyError: 'pkg_id'
 
+        Cleanup:
+
+            >>> transaction.abort()
+            >>> c.close()
+
         """
         if self.timeout == 0:
-            return IterableUserDict.__getitem__(self, pkg_id)
+            return UserDict.__getitem__(self, pkg_id)
 
         now = time.time()
 
@@ -239,7 +275,7 @@ class PersistentSessionDataContainer(zope.location.Location,
                 self._v_old_sweep = self._v_last_sweep
             self._v_last_sweep = now
 
-        rv = IterableUserDict.__getitem__(self, pkg_id)
+        rv = UserDict.__getitem__(self, pkg_id)
         # Only update the lastAccessTime once every few minutes, rather than
         # every hit, to avoid ZODB bloat and conflicts
         if rv.getLastAccessTime() + self.resolution < now:
@@ -267,7 +303,7 @@ class PersistentSessionDataContainer(zope.location.Location,
 
         """
         session_data.setLastAccessTime(int(time.time()))
-        return IterableUserDict.__setitem__(self, pkg_id, session_data)
+        return UserDict.__setitem__(self, pkg_id, session_data)
 
     def sweep(self):
         """Clean out stale data
@@ -334,12 +370,12 @@ class RAMSessionDataContainer(PersistentSessionDataContainer):
     def _getData(self):
 
         # Open a connection to _ram_storage per thread
-        tid = thread.get_ident()
-        if not self._conns.has_key(tid):
+        tid = get_ident()
+        if tid not in self._conns:
             self._conns[tid] = self._ram_db.open()
 
         root = self._conns[tid].root()
-        if not root.has_key(self.key):
+        if self.key not in root:
             root[self.key] = OOBTree()
         return root[self.key]
 
@@ -350,10 +386,10 @@ class RAMSessionDataContainer(PersistentSessionDataContainer):
         self._ram_db.pack(time.time())
 
 
+@zope.interface.implementer(ISession)
+@zope.component.adapter(IRequest)
 class Session(object):
     """See zope.session.interfaces.ISession"""
-    zope.interface.implements(ISession)
-    zope.component.adapts(IRequest)
 
     def __init__(self, request):
         self.client_id = str(IClientId(request))
@@ -370,7 +406,7 @@ class Session(object):
 
         """See zope.session.interfaces.ISession
 
-            >>> import tests
+            >>> from zope.session import tests
             >>> request = tests.setUp(PersistentSessionDataContainer)
 
            If we use get we get None or default returned if the pkg_id
@@ -414,9 +450,10 @@ class Session(object):
     def __getitem__(self, pkg_id):
         """See zope.session.interfaces.ISession
 
-            >>> import tests
+            >>> from zope.publisher.http import HTTPRequest
+            >>> from zope.session import tests
             >>> request = tests.setUp(PersistentSessionDataContainer)
-            >>> request2 = tests.HTTPRequest(StringIO(''), {}, None)
+            >>> request2 = HTTPRequest(StringIO(''), {}, None)
 
             >>> ISession.providedBy(Session(request))
             True
@@ -474,7 +511,8 @@ class Session(object):
         raise NotImplementedError
 
 
-class SessionData(persistent.Persistent, IterableUserDict):
+@zope.interface.implementer(ISessionData)
+class SessionData(persistent.Persistent, UserDict):
     """See zope.session.interfaces.ISessionData
 
         >>> session = SessionData()
@@ -520,7 +558,6 @@ class SessionData(persistent.Persistent, IterableUserDict):
         13
 
     """
-    zope.interface.implements(ISessionData)
 
     # this is for support of legacy sessions; this comment and
     # the next line will be removed in a later release
@@ -552,13 +589,13 @@ class SessionData(persistent.Persistent, IterableUserDict):
                               doc='integer value of the last access time')
 
 
-class SessionPkgData(persistent.Persistent, IterableUserDict):
+@zope.interface.implementer(ISessionPkgData)
+class SessionPkgData(persistent.Persistent, UserDict):
     """See zope.session.interfaces.ISessionPkgData
 
         >>> session = SessionPkgData()
         >>> ISessionPkgData.providedBy(session)
         True
     """
-    zope.interface.implements(ISessionPkgData)
     def __init__(self):
         self.data = OOBTree()
