@@ -13,55 +13,59 @@
 ##############################################################################
 """Session tests
 """
-from cStringIO import StringIO
-from zope.testing import cleanup
-import doctest
-import os
-import os.path
-import transaction
 import unittest
-import zope.component
+import os
 
-from zope.component import provideHandler, getGlobalSiteManager
-from zope.session.interfaces import IClientId, IClientIdManager, ISession
-from zope.session.interfaces import ISessionDataContainer
-from zope.session.interfaces import ISessionPkgData, ISessionData
-from zope.session.session import ClientId, Session
-from zope.session.session import PersistentSessionDataContainer
-from zope.session.session import RAMSessionDataContainer
-from zope.session.http import CookieClientIdManager
 
-from zope.publisher.interfaces import IRequest
-from zope.publisher.http import HTTPRequest
-
-def setUp(session_data_container_class=PersistentSessionDataContainer):
+def setUp(session_data_container_class=None):
+    from io import StringIO
+    from zope.testing import cleanup
+    from zope.component import provideAdapter
+    from zope.component import provideUtility
+    from zope.publisher.http import HTTPRequest
+    from zope.publisher.interfaces import IRequest
+    from zope.session.http import CookieClientIdManager
+    from zope.session.interfaces import IClientId
+    from zope.session.interfaces import IClientIdManager
+    from zope.session.interfaces import ISession
+    from zope.session.interfaces import ISessionDataContainer
+    from zope.session.session import ClientId
+    from zope.session.session import PersistentSessionDataContainer
+    from zope.session.session import Session
     cleanup.setUp()
-    zope.component.provideAdapter(ClientId, (IRequest,), IClientId)
-    zope.component.provideAdapter(Session, (IRequest,), ISession)
-    zope.component.provideUtility(CookieClientIdManager(), IClientIdManager)
+    if session_data_container_class is None:
+        session_data_container_class = PersistentSessionDataContainer
+    provideAdapter(ClientId, (IRequest,), IClientId)
+    provideAdapter(Session, (IRequest,), ISession)
+    provideUtility(CookieClientIdManager(), IClientIdManager)
     sdc = session_data_container_class()
     for product_id in ('', 'products.foo', 'products.bar', 'products.baz'):
-        zope.component.provideUtility(sdc, ISessionDataContainer, product_id)
+        provideUtility(sdc, ISessionDataContainer, product_id)
     request = HTTPRequest(StringIO(), {}, None)
     return request
 
 def tearDown():
+    from zope.testing import cleanup
     cleanup.tearDown()
 
 # Test the code in our API documentation is correct
 def test_documentation():
     pass
-test_documentation.__doc__ = '''
+
+with open(os.path.join(os.path.dirname(__file__), 'api.txt')) as f:
+    test_documentation.__doc__ = '''
+    >>> from zope.session.session import RAMSessionDataContainer
     >>> request = setUp(RAMSessionDataContainer)
 
     %s
 
     >>> tearDown()
 
-    ''' % (open(os.path.join(os.path.dirname(__file__), 'api.txt')).read(),)
+    ''' % f.read()
 
 
 def tearDownTransaction(test):
+    import transaction
     transaction.abort()
 
 
@@ -80,10 +84,13 @@ def testConflicts():
     ...     from ZODB.DB import DB
     ...     from ZODB.tests.util import ConflictResolvingMappingStorage
     ...     db = DB(ConflictResolvingMappingStorage())
+    ...     tmpdir = None
     ... except ImportError:
     ...     # ZODB 3.9 (ConflictResolvingMappingStorage no longer exists)
+    ...     import tempfile
     ...     import ZODB.DB
-    ...     db = ZODB.DB('Data.fs')
+    ...     tmpdir = tempfile.mkdtemp(prefix='zope.session-', suffix='-test')
+    ...     db = ZODB.DB(os.path.join(tmpdir, 'testConflicts-Data.fs'))
     >>> from zope.session.session import (
     ...     PersistentSessionDataContainer, SessionData)
     >>> import transaction
@@ -125,6 +132,15 @@ def testConflicts():
     >>> tm_B.commit()
 
     Q.E.D.
+
+    Cleanup::
+
+    >>> conn_A.close()
+    >>> conn_B.close()
+    >>> db.close()
+    >>> if tmpdir:
+    ...     import shutil
+    ...     shutil.rmtree(tmpdir)
     """
 
 def testSessionIterationBug():
@@ -156,12 +172,35 @@ def testSessionIterationBug():
 
 
 def test_suite():
+    import doctest
+    import re
+    from zope.testing import renormalizing
+
+    checker = renormalizing.RENormalizing([
+        # Python 3 strings remove the "u".
+        (re.compile("u('.*?')"),
+        r"\1"),
+        (re.compile('u(".*?")'),
+        r"\1"),
+        # Python 3 bytes add a "b".
+        (re.compile("b('.*?')"),
+        r"\1"),
+        (re.compile('b(".*?")'),
+        r"\1"),
+        # Python 3 adds module name to exceptions.
+        (re.compile("zope.session.http.MissingClientIdException"),
+        r"MissingClientIdException"),
+        ])
     suite = unittest.TestSuite()
-    suite.addTest(doctest.DocTestSuite())
-    suite.addTest(doctest.DocTestSuite('zope.session.session',
-        tearDown=tearDownTransaction))
-    suite.addTest(doctest.DocTestSuite('zope.session.http',
-        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,)
+    suite.addTest(doctest.DocTestSuite(
+            checker=checker,
+            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,))
+    suite.addTest(doctest.DocTestSuite(
+            'zope.session.session', checker=checker,
+            tearDown=tearDownTransaction))
+    suite.addTest(doctest.DocTestSuite(
+            'zope.session.http', checker=checker,
+            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,)
         )
     return suite
 
